import asyncio
import logging
import os
import shutil
from pathlib import Path

import dataiku
from dataiku.customrecipe import (
    get_input_names_for_role,
    get_output_names_for_role,
    get_recipe_config,
    get_recipe_resource,
)
from dku_graphrag.index.dataiku_graph_index_builder import DataikuGraphragIndexBuilder
from graphrag.config.load_config import load_config

config = get_recipe_config()
text_column = config.get("text_column")
source_column = config.get("source_column")
title_column = config.get("title_column")
attribute_columns = config.get("attribute_columns", [])
chunk_size = config.get("chunk_size")
chunk_overlap = config.get("chunk_overlap")
group_by_columns = config.get("group_by_column", [])
verbose_mode = config.get("verbose_mode", False)
chat_completion_llm_id = config.get("chat_completion_llm_id")
embedding_llm_id = config.get("embedding_llm_id")

logger = logging.getLogger("graphrag recipe")
logging.basicConfig(
    level=logging.DEBUG if verbose_mode else logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    force=True,
)


input_dataset_name = get_input_names_for_role("input_dataset")[0]
input_dataset = dataiku.Dataset(input_dataset_name)

output_folder_name = get_output_names_for_role("output_folder")[0]
output_folder = dataiku.Folder(output_folder_name)
output_folder_path = output_folder.get_path()
prompts_folder_path = os.path.join(output_folder_path, "prompts")

if os.path.exists(output_folder_path):
    shutil.rmtree(output_folder_path)


resource_folder_path = get_recipe_resource()
settings_source = os.path.join(resource_folder_path, "settings.yaml")
settings_target = os.path.join(output_folder_path, "settings.yaml")

if os.path.exists(settings_target):
    logger.debug("Deleting old settings.yaml file")
    os.remove(settings_target)

with output_folder.get_writer("settings.yaml") as writer:
    with open(settings_source, "rb") as f:
        writer.write(f.read())
    logger.debug(f"Copied new settings.yaml file from resources to {settings_target}")


prompts_source = os.path.join(resource_folder_path, "prompts")
prompts_target = os.path.join(output_folder_path, "prompts")

if os.path.exists(prompts_target):
    logger.debug("Deleting old prompts folder")
    shutil.rmtree(prompts_target)

logger.debug("Copying new prompts from plugin")
shutil.copytree(prompts_source, prompts_target)

input_dir = os.path.join(output_folder_path, "input")

logger.debug("Deleting old input directory")
if os.path.exists(input_dir):
    shutil.rmtree(input_dir)

os.makedirs(input_dir)
logger.debug("Created input directory")


def update_prompt(file_name: str, content_value: str) -> None:
    prompt_file_path = os.path.join(prompts_folder_path, file_name)
    try:
        if not prompt_file_path:
            logger.debug(f"Deleting prompt_file {file_name}")
            os.remove(prompt_file_path)
        with open(prompt_file_path, "w", encoding="utf-8") as writer:
            writer.write(content_value)
        logger.debug(f"Updated prompt {file_name}")
    except OSError as e:
        logger.exception(f"Failed to update prompt {file_name}: {e}")
    except Exception as e:  # Catches unexpected errors
        logger.exception(f"Unexpected error updating prompt {file_name}: {e}")


selected_columns = [text_column] + attribute_columns
logger.info(f"Input dataset selected_columns: {selected_columns}")

df = input_dataset.get_dataframe(columns=selected_columns)
input_file_path = os.path.join(input_dir, f"{input_dataset_name}.csv")
df.to_csv(input_file_path, index=False, encoding="utf-8")

logger.debug(f"Input dataset copied to indexing folder: {input_file_path}")


logger.debug(f"Selected columns for indexing: {selected_columns}")
logger.debug(f"DataFrame shape: {df.shape}")

root_dir = Path(output_folder_path)
graph_rag_config = load_config(root_dir, None)


if "db_uri" in graph_rag_config.embeddings.vector_store:
    graph_rag_config.embeddings.vector_store["db_uri"] = root_dir / graph_rag_config.embeddings.vector_store["db_uri"]
    logger.info(f"Setting vector db absolite path: {graph_rag_config.embeddings.vector_store['db_uri']}")
else:
    logger.error("Error: 'db_uri' key not found in vector_store")
if graph_rag_config.input:
    if selected_columns:
        graph_rag_config.input.document_attribute_columns = selected_columns
    if text_column:
        graph_rag_config.input.text_column = text_column
    if title_column:
        graph_rag_config.input.title_column = title_column
    if source_column:
        graph_rag_config.input.source_column = source_column
if graph_rag_config.chunks:
    graph_rag_config.chunks.size = chunk_size
    graph_rag_config.chunks.overlap = chunk_overlap
    if group_by_columns and len(group_by_columns) > 0:
        graph_rag_config.chunks.group_by_columns = group_by_columns

if config.get("embed_graph_enabled", False):
    graph_rag_config.embed_graph.num_walks = config.get("num_walks")
    graph_rag_config.embed_graph.walk_length = config.get("walk_length")
    graph_rag_config.embed_graph.window_size = config.get("window_size")
    graph_rag_config.embed_graph.iterations = config.get("iterations")
    graph_rag_config.embed_graph.random_seed = config.get("random_seed")


update_prompt("entity_extraction.txt", config.get("entity_extraction_prompt"))
graph_rag_config.entity_extraction.entity_types = config.get("entity_types")

update_prompt("summarize_descriptions.txt", config.get("summarize_descriptions_prompt"))
update_prompt("community_report.txt", config.get("community_report_prompt"))
update_prompt("claim_extraction.txt", config.get("  claim_extraction_prompt"))


builder = DataikuGraphragIndexBuilder(chat_completion_llm_id, embedding_llm_id)
asyncio.run(
    builder.run_build_index_pipeline(config=graph_rag_config, verbose=verbose_mode, resume=None, memprofile=False)
)
